package com.example.sync2chandroidsample;

import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.StreamCorruptedException;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import android.app.AlertDialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.provider.OpenableColumns;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

/**
 * A placeholder fragment containing a simple view.
 */
public class PlaceholderFragment extends Fragment {

	private static final String THREADS_SAVE_FILE_NAME = "threads.json";
	private View mRootView;
	private Handler mHandler = new Handler();

	public PlaceholderFragment() {
	}

	@Override
	public View onCreateView(LayoutInflater inflater, ViewGroup container,
			Bundle savedInstanceState) {
		mRootView = inflater.inflate(R.layout.fragment_main, container, false);

		SharedPreferences sh = PreferenceManager
				.getDefaultSharedPreferences(getActivity());
		EditText idText = (EditText) mRootView.findViewById(R.id.editText1);
		EditText passText = (EditText) mRootView.findViewById(R.id.editText2);
		idText.setText(sh.getString("sync2ch_id", ""));
		passText.setText(sh.getString("sync2ch_pass", ""));

		List<ThreadInfo> favoriteThreads = loadThreadsFromFile();
		showFavoriteThreads(favoriteThreads);

		Button syncButton = (Button) mRootView.findViewById(R.id.button1);
		syncButton.setOnClickListener(new Button.OnClickListener() {
			@Override
			public void onClick(View v) {
				EditText idText = (EditText) mRootView
						.findViewById(R.id.editText1);
				EditText passText = (EditText) mRootView
						.findViewById(R.id.editText2);
				final String id = idText.getText().toString();
				final String pass = passText.getText().toString();

				SharedPreferences sh = PreferenceManager
						.getDefaultSharedPreferences(getActivity());

				sh.edit().putString("sync2ch_id", id)
						.putString("sync2ch_pass", pass).commit();

				if (id == null || pass == null) {

					Toast.makeText(getActivity(), "IDとAPI接続用パスワードを入力してください",
							Toast.LENGTH_LONG).show();
				} else {
					new Thread(new Runnable() {
						@Override
						public void run() {
							try {

								startSync(id, pass);

							} catch (IOException e) {
								e.printStackTrace();
							} catch (ParserConfigurationException e) {
								e.printStackTrace();
							} catch (SAXException e) {
								e.printStackTrace();
							}
						}
					}).start();

				}

			}
		});
		return mRootView;
	}

	private List<ThreadInfo> loadThreadsFromFile() {

		try {
			ObjectInputStream ois = new ObjectInputStream(
					getActivity().openFileInput(THREADS_SAVE_FILE_NAME
							));
			return (List<ThreadInfo>) ois.readObject();
		} catch (StreamCorruptedException e) {
			e.printStackTrace();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		return new ArrayList<ThreadInfo>();
	}

	private void saveThreads(List<ThreadInfo> threads) {
		try {
			ObjectOutputStream oos = new ObjectOutputStream(getActivity()
					.openFileOutput(THREADS_SAVE_FILE_NAME,
							Context.MODE_PRIVATE));
			oos.writeObject(threads);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static String escape(String str) {
		StringBuffer sb = new StringBuffer();

		if (str != null) {
			for (int i = 0; i < str.length(); i++) {
				if (str.charAt(i) == '<') {
					sb.append("&lt;");
				} else if (str.charAt(i) == '>') {
					sb.append("&gt;");
				} else if (str.charAt(i) == '"') {
					sb.append("&quot;");
				} else if (str.charAt(i) == '&') {
					sb.append("&amp;");
				} else if (str.charAt(i) == '\'') {
					sb.append("&apos;");
				} else {
					sb.append(str.charAt(i));
				}
			}
		}
		return sb.toString();
	}

	private void startSync(String id2, String pass) throws IOException,
			ParserConfigurationException, SAXException {

		SharedPreferences sh = PreferenceManager
				.getDefaultSharedPreferences(getActivity());

		Log.d("id2 = ", id2);
		Log.d("pass = ", pass);

		final String syncNumber = sh.getString("sync_number", "0");
		final String clientId = sh.getString("client_id", "0");

		URL url = new URL("https://sync2ch.com/api/sync3");

		HttpURLConnection httpUrlConnection;

		httpUrlConnection = (HttpURLConnection) url.openConnection();
		httpUrlConnection.setDoOutput(true);

		httpUrlConnection.setRequestProperty("Content-Type", "text/xml");
		httpUrlConnection.setRequestProperty("Encoding", "gzip");

		byte[] encode = android.util.Base64.encode(
				(id2 + ":" + pass).getBytes(), 0);

		httpUrlConnection.setRequestProperty("Authorization", "Basic "
				+ new String(encode));

		httpUrlConnection.setRequestProperty("User-Agent",
				"Sync2chAndroidSample");
		OutputStream os = httpUrlConnection.getOutputStream();
		os = new java.util.zip.GZIPOutputStream(os);

		OutputStreamWriter writer = new OutputStreamWriter(os, "UTF-8");

		BufferedWriter bw = new BufferedWriter(writer);
		bw.write("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n");

		String version = "1.1.1.1";
		bw.write("<sync2ch_request sync_number=\"" + syncNumber
				+ "\" client_name=\"Sync2chAndroidSample\" client_id=\""
				+ clientId + "\" client_version=\"" + version
				+ "\" os=\"Android " + escape(android.os.Build.VERSION.RELEASE)
				+ "\" " + ">");

		List<ThreadInfo> favoriteThreads = loadThreadsFromFile();

		bw.write("<thread_group  category='favorite' struct='Sync2chAndroidSample'>");

		for (ThreadInfo th : favoriteThreads) {
			bw.write("<th url=\"" + th.getUrl() + "\" title=\""
					+ escape(th.getTitle()) + "\" read=\"" + th.getReadPos()
					+ "\" now=\"" + th.getCurrentPos() + "\"" + " count=\""
					+ th.getResCount() + "\"/>");
		}

		bw.write("</thread_group>\n");
		bw.write("</sync2ch_request>");

		bw.close();

		int responseCode = httpUrlConnection.getResponseCode();
		if (responseCode != 200) {
			Log.d("responseCode", "" + responseCode);
			return;
		}

		InputStream inputStream = httpUrlConnection.getInputStream();

		String contentEncoding = httpUrlConnection
				.getHeaderField("Content-Encoding");

		if (contentEncoding == "gzip") {
			inputStream = new java.util.zip.GZIPInputStream(inputStream);
		}

		parseResponseInputStream(favoriteThreads, inputStream);

		mHandler.post(new Runnable() {

			@Override
			public void run() {

				Toast.makeText(getActivity(), "同期終了", Toast.LENGTH_LONG).show();

			}
		});

	}

	private void parseResponseInputStream(List<ThreadInfo> favoriteThreads,
			InputStream inputStream) throws ParserConfigurationException,
			SAXException, IOException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = factory.newDocumentBuilder();

		Document doc = builder.parse(inputStream);

		Element sync2chResponse = doc.getDocumentElement();

		if (sync2chResponse == null) {
			return;
		}

		HashMap<String, Element> idEntities = new HashMap<String, Element>();
		HashMap<String, ThreadInfo> urlThreadInfoMap = new HashMap<String, ThreadInfo>();
		for (ThreadInfo th : favoriteThreads) {
			urlThreadInfoMap.put(th.getUrl(), th);
		}

		NodeList nodes = sync2chResponse.getChildNodes();
		if (nodes != null) {

			for (int i = 0; i < nodes.getLength(); i++) {
				Node item = nodes.item(i);
				if (item.getNodeType() != Node.ELEMENT_NODE) {
					continue;
				}

				final Element groupElement = (Element) item;

				String tagName = groupElement.getTagName();

				if ("entities".equals(tagName)) {

					NodeList threadNodes = groupElement.getChildNodes();
					for (int j = 0; j < threadNodes.getLength(); j++) {
						Node item2 = threadNodes.item(j);
						if (item2.getNodeType() != Node.ELEMENT_NODE)
							continue;

						
						Log.d("entity", "entity");
						Element thElement = (Element) item2;
						if (thElement.getTagName().equals("th")) {
							String id = thElement.getAttribute("id");
							idEntities.put(id, thElement);
						}
					}

				} else if ("thread_group".equals(tagName)) {

					String category = groupElement.getAttribute("category");
					Log.d("cateogry", category);
					if ("favorite".equals(category)) {

						final ArrayList<ThreadInfo> newFavoriteThreads = new ArrayList<ThreadInfo>();
						NodeList threadNodes = groupElement.getChildNodes();
						for (int j = 0; j < threadNodes.getLength(); j++) {
							Node item2 = threadNodes.item(j);
							if (item2.getNodeType() != Node.ELEMENT_NODE)
								continue;

							Element thElement = (Element) item2;
							
							Log.d("tag", thElement.getTagName());
							if (thElement.getTagName().equals("th")) {
								String id = thElement.getAttribute("id");
								Element entity = (Element) idEntities.get(id);

								
								
								
								String thUrl = entity.getAttribute("url");
								String title = entity.getAttribute("title");
								String readPos = entity.getAttribute("read");
								String currentPos = entity.getAttribute("now");
								String count = entity.getAttribute("count");
								Log.d(thUrl, title);
								
								ThreadInfo newThreadInfo = null;
								if (urlThreadInfoMap.containsKey(thUrl)) {
									newThreadInfo = urlThreadInfoMap.get(thUrl);
								} else {
									newThreadInfo = new ThreadInfo();
									newThreadInfo.setUrl(thUrl);
								}

								String status = entity.getAttribute("s");
								if (status.equals("n") == false) {
									newThreadInfo.setTitle(title);
									newThreadInfo.setReadPos(Integer
											.parseInt(readPos));
									newThreadInfo.setCurrentPos(Integer
											.parseInt(currentPos));
									newThreadInfo.setResCount(Integer
											.parseInt(count));
								}

								newFavoriteThreads.add(newThreadInfo);

							}
						}

						saveThreads(newFavoriteThreads);
						mHandler.post(new Runnable() {
							@Override
							public void run() {
								showFavoriteThreads(newFavoriteThreads);
							}
						});

					}

				}
			}
		}
	}

	private void showFavoriteThreads(final List<ThreadInfo> favoriteThreads) {

		ListView listView = (ListView) mRootView.findViewById(R.id.listView1);

		ArrayAdapter<ThreadInfo> adapter = new ArrayAdapter<ThreadInfo>(
				getActivity(), android.R.layout.simple_list_item_1,
				favoriteThreads) {

			public View getView(int position, View convertView, ViewGroup parent) {
				LayoutInflater inflater = getActivity().getLayoutInflater();
				View root = inflater.inflate(R.layout.threadinfo_layout, null);

				ThreadInfo thInfo = favoriteThreads.get(position);

				TextView titleTextView = (TextView) root
						.findViewById(R.id.textView1);
				TextView urlTextView = (TextView) root
						.findViewById(R.id.textView2);
				TextView metaTextView = (TextView) root
						.findViewById(R.id.textView3);

				titleTextView.setText(thInfo.getTitle());
				urlTextView.setText(thInfo.getUrl());

				String metaInfo = "現在:" + thInfo.getCurrentPos() + " 既読:"
						+ thInfo.getReadPos() + " レス数:" + thInfo.getResCount();
				metaTextView.setText(metaInfo);

				return root;
			};
		};

		listView.setAdapter(adapter);

	}
}